#ifndef __except_h
#define __except_h

/*
 * SPDX-FileCopyrightText: 2001-2003  Daniel Kelly
 * SPDX-FileCopyrightText: Peter Pentchev <roam@ringlet.net>
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

void set_exceptions(const char *exp, int octet[4][256]);
bool except(uint32_t *current, int octet[4][256], int increment);

#endif /* __except_h */
